/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 *
 */

import * as t from '@babel/types';
import type { PluginPass } from '@babel/core';
import type { NodePath } from '@babel/traverse';
import type {
  CompiledNamespaces,
  StyleXOptions as RuntimeOptions,
} from '@stylexjs/shared';
import type { ImportOptions } from '@babel/helper-module-imports';
type ImportAdditionOptions = Omit<
  Partial<ImportOptions>,
  'ensureLiveReference' | 'ensureNoContext'
>;
export type ImportPathResolution =
  | false
  | ['themeNameRef' | 'filePath', string];
type ModuleResolution =
  | Readonly<{
      type: 'commonJS';
      rootDir?: string;
      themeFileExtension?: null | undefined | string;
    }>
  | Readonly<{ type: 'haste'; themeFileExtension?: null | undefined | string }>
  | Readonly<{
      type: 'experimental_crossFileParsing';
      rootDir?: string;
      themeFileExtension?: null | undefined | string;
    }>;
export type StyleXOptions = Readonly<
  Omit<
    RuntimeOptions,
    keyof ({
      importSources: ReadonlyArray<
        string | Readonly<{ from: string; as: string }>
      >;
      runtimeInjection:
        | boolean
        | (null | undefined | string)
        | Readonly<{ from: string; as: string }>;
      treeshakeCompensation?: boolean;
      genConditionalClasses: boolean;
      unstable_moduleResolution: null | undefined | ModuleResolution;
      aliases?:
        | null
        | undefined
        | Readonly<{ [$$Key$$: string]: string | ReadonlyArray<string> }>;
    })
  > & {
    importSources: ReadonlyArray<
      string | Readonly<{ from: string; as: string }>
    >;
    runtimeInjection:
      | boolean
      | (null | undefined | string)
      | Readonly<{ from: string; as: string }>;
    treeshakeCompensation?: boolean;
    genConditionalClasses: boolean;
    unstable_moduleResolution: null | undefined | ModuleResolution;
    aliases?:
      | null
      | undefined
      | Readonly<{ [$$Key$$: string]: string | ReadonlyArray<string> }>;
  }
>;
type StyleXStateOptions = Readonly<
  Omit<
    StyleXOptions,
    keyof ({
      runtimeInjection:
        | (null | undefined | string)
        | Readonly<{ from: string; as: null | undefined | string }>;
      aliases?:
        | null
        | undefined
        | Readonly<{ [$$Key$$: string]: ReadonlyArray<string> }>;
    })
  > & {
    runtimeInjection:
      | (null | undefined | string)
      | Readonly<{ from: string; as: null | undefined | string }>;
    aliases?:
      | null
      | undefined
      | Readonly<{ [$$Key$$: string]: ReadonlyArray<string> }>;
  }
>;
declare class StateManager {
  readonly _state: PluginPass;
  readonly importPaths: Set<string>;
  readonly stylexImport: Set<string>;
  readonly stylexPropsImport: Set<string>;
  readonly stylexAttrsImport: Set<string>;
  readonly stylexCreateImport: Set<string>;
  readonly stylexIncludeImport: Set<string>;
  readonly stylexFirstThatWorksImport: Set<string>;
  readonly stylexKeyframesImport: Set<string>;
  readonly stylexDefineVarsImport: Set<string>;
  readonly stylexCreateThemeImport: Set<string>;
  readonly stylexTypesImport: Set<string>;
  injectImportInserted: null | undefined | t.Identifier;
  readonly styleMap: Map<string, CompiledNamespaces>;
  readonly styleVars: Map<string, NodePath>;
  readonly styleVarsToKeep: Set<[string, true | string, true | Array<string>]>;
  inStyleXCreate: boolean;
  readonly options: StyleXStateOptions;
  constructor(state: PluginPass);
  setOptions(options: {
    readonly [$$Key$$: string]: unknown;
  }): StyleXStateOptions;
  get importPathString(): string;
  get importSources(): ReadonlyArray<string>;
  importAs(source: string): null | string;
  get canReferenceTheme(): boolean;
  get metadata(): { [key: string]: any };
  get runtimeInjection():
    | null
    | undefined
    | Readonly<{ from: string; as?: null | undefined | string }>;
  addNamedImport(
    statementPath: NodePath,
    as: string,
    from: string,
    options: ImportAdditionOptions,
  ): t.Identifier;
  addDefaultImport(
    statementPath: NodePath,
    from: string,
    options: ImportAdditionOptions,
  ): t.Identifier;
  get isDev(): boolean;
  get isTest(): boolean;
  get filename(): string | void;
  get cssVars(): any;
  get treeshakeCompensation(): boolean;
  get fileNameForHashing(): null | string;
  getPackageNameAndPath(
    filepath: string,
  ): null | readonly [packageName: string, packageDir: string];
  getCanonicalFilePath(filePath: string): string;
  importPathResolver(importPath: string): ImportPathResolution;
  addStyle(style: [string, { ltr: string; rtl?: string | null }, number]): void;
  registerStyles(
    styles: ReadonlyArray<
      [string, { ltr: string; rtl?: string | null }, number]
    >,
    path?: null | undefined | NodePath,
  ): void;
  markComposedNamespace(
    memberExpression: [string, true | string, true | Array<string>],
  ): void;
}
export default StateManager;
